@@ -35,9 +35,6 @@
 #include "gstvaapiobject_priv.h"
 
 #include "gstvaapicompat.h"
-#ifdef HAVE_VA_VA_DEC_VP8_H
-#include <va/va_dec_vp8.h>
-#endif
 
 #define DEBUG 1
 #include "gstvaapidebug.h"
