@@ -43,10 +43,11 @@ GST_DEBUG_CATEGORY (gst_debug_vaapi);
 GST_DEBUG_CATEGORY (gst_debug_vaapi_display);
 #define GST_CAT_DEFAULT gst_debug_vaapi_display
 
-#define _do_init \
-    G_ADD_PRIVATE (GstVaapiDisplay); \
-    GST_DEBUG_CATEGORY_INIT (gst_debug_vaapi_display, \
-        "vaapidisplay", 0, "VA-API Display");
+#define _do_init                                        \
+    G_ADD_PRIVATE (GstVaapiDisplay);                    \
+    GST_DEBUG_CATEGORY_INIT (gst_debug_vaapi_display,   \
+        "vaapidisplay", 0, "VA-API Display");           \
+    GST_DEBUG_CATEGORY_INIT (gst_debug_vaapi, "vaapi", 0, "VA-API helper");
 
 G_DEFINE_TYPE_WITH_CODE (GstVaapiDisplay, gst_vaapi_display, GST_TYPE_OBJECT,
     _do_init);
@@ -83,9 +84,7 @@ struct _GstVaapiFormatInfo
 
 enum
 {
-  PROP_0,
-
-  PROP_RENDER_MODE,
+  PROP_RENDER_MODE = 1,
   PROP_ROTATION,
   PROP_HUE,
   PROP_SATURATION,
@@ -95,13 +94,8 @@ enum
   N_PROPERTIES
 };
 
-static GstVaapiDisplayCache *g_display_cache;
-static GMutex g_display_cache_lock;
-
 static GParamSpec *g_properties[N_PROPERTIES] = { NULL, };
 
-static void gst_vaapi_display_properties_init (void);
-
 static gboolean
 get_attribute (GstVaapiDisplay * display, VADisplayAttribType type,
     gint * value);
@@ -115,44 +109,6 @@ get_color_balance (GstVaapiDisplay * display, guint prop_id, gfloat * v);
 static gboolean
 set_color_balance (GstVaapiDisplay * display, guint prop_id, gfloat v);
 
-static void
-libgstvaapi_init_once (void)
-{
-  static gsize g_once = FALSE;
-
-  if (!g_once_init_enter (&g_once))
-    return;
-
-  GST_DEBUG_CATEGORY_INIT (gst_debug_vaapi, "vaapi", 0, "VA-API helper");
-
-  gst_vaapi_display_properties_init ();
-
-  g_once_init_leave (&g_once, TRUE);
-}
-
-static GstVaapiDisplayCache *
-get_display_cache (void)
-{
-  GstVaapiDisplayCache *cache = NULL;
-
-  g_mutex_lock (&g_display_cache_lock);
-  if (!g_display_cache)
-    g_display_cache = gst_vaapi_display_cache_new ();
-  if (g_display_cache)
-    cache = gst_vaapi_display_cache_ref (g_display_cache);
-  g_mutex_unlock (&g_display_cache_lock);
-  return cache;
-}
-
-static void
-free_display_cache (void)
-{
-  g_mutex_lock (&g_display_cache_lock);
-  if (g_display_cache && gst_vaapi_display_cache_is_empty (g_display_cache))
-    gst_vaapi_display_cache_replace (&g_display_cache, NULL);
-  g_mutex_unlock (&g_display_cache_lock);
-}
-
 /* GstVaapiDisplayType enumerations */
 GType
 gst_vaapi_display_type_get_type (void)
@@ -235,13 +191,13 @@ append_formats (GArray * formats, const VAImageFormat * va_formats,
     guint * flags, guint n)
 {
   GstVideoFormat format;
-  const GstVaapiFormatInfo *YV12_fip = NULL;
-  const GstVaapiFormatInfo *I420_fip = NULL;
+  int YV12_idx = -1;
+  int I420_idx = -1;
+  const GstVaapiFormatInfo *fip;
   guint i;
 
   for (i = 0; i < n; i++) {
     const VAImageFormat *const va_format = &va_formats[i];
-    const GstVaapiFormatInfo **fipp;
 
     format = gst_vaapi_video_format_from_va_format (va_format);
     if (format == GST_VIDEO_FORMAT_UNKNOWN) {
@@ -253,25 +209,25 @@ append_formats (GArray * formats, const VAImageFormat * va_formats,
 
     switch (format) {
       case GST_VIDEO_FORMAT_YV12:
-        fipp = &YV12_fip;
+        YV12_idx = formats->len - 1;
         break;
       case GST_VIDEO_FORMAT_I420:
-        fipp = &I420_fip;
+        I420_idx = formats->len - 1;
         break;
       default:
-        fipp = NULL;
         break;
     }
-    if (fipp)
-      *fipp = &g_array_index (formats, GstVaapiFormatInfo, formats->len - 1);
   }
 
   /* Append I420 (resp. YV12) format if YV12 (resp. I420) is not
      supported by the underlying driver */
-  if (YV12_fip && !I420_fip)
-    append_format (formats, GST_VIDEO_FORMAT_I420, YV12_fip->flags);
-  else if (I420_fip && !YV12_fip)
-    append_format (formats, GST_VIDEO_FORMAT_YV12, I420_fip->flags);
+  if ((YV12_idx != -1) && (I420_idx == -1)) {
+    fip = &g_array_index (formats, GstVaapiFormatInfo, YV12_idx);
+    append_format (formats, GST_VIDEO_FORMAT_I420, fip->flags);
+  } else if ((I420_idx != -1) && (YV12_idx == -1)) {
+    fip = &g_array_index (formats, GstVaapiFormatInfo, I420_idx);
+    append_format (formats, GST_VIDEO_FORMAT_YV12, fip->flags);
+  }
 }
 
 /* Sort image formats. Prefer YUV formats first */
@@ -570,6 +526,7 @@ ensure_profiles (GstVaapiDisplay * display)
         case GST_VAAPI_ENTRYPOINT_SLICE_ENCODE:
         case GST_VAAPI_ENTRYPOINT_PICTURE_ENCODE:
         case GST_VAAPI_ENTRYPOINT_SLICE_ENCODE_LP:
+        case GST_VAAPI_ENTRYPOINT_SLICE_ENCODE_FEI:
           g_array_append_val (priv->encoders, config);
           break;
       }
@@ -825,6 +782,25 @@ gst_vaapi_display_calculate_pixel_aspect_ratio (GstVaapiDisplay * display)
   priv->par_d = par[index][windex ^ 1];
 }
 
+static void
+gst_vaapi_display_ensure_screen_resolution (GstVaapiDisplay * display)
+{
+  GstVaapiDisplayPrivate *const priv = GST_VAAPI_DISPLAY_GET_PRIVATE (display);
+  const GstVaapiDisplayClass *const klass =
+      GST_VAAPI_DISPLAY_GET_CLASS (display);
+
+  if (priv->got_scrres)
+    return;
+
+  if (klass->get_size)
+    klass->get_size (display, &priv->width, &priv->height);
+  if (klass->get_size_mm)
+    klass->get_size_mm (display, &priv->width_mm, &priv->height_mm);
+
+  gst_vaapi_display_calculate_pixel_aspect_ratio (display);
+  priv->got_scrres = TRUE;
+}
+
 static void
 gst_vaapi_display_destroy (GstVaapiDisplay * display)
 {
@@ -856,8 +832,7 @@ gst_vaapi_display_destroy (GstVaapiDisplay * display)
   }
 
   if (priv->display) {
-    if (!priv->parent)
-      vaTerminate (priv->display);
+    vaTerminate (priv->display);
     priv->display = NULL;
   }
 
@@ -872,78 +847,54 @@ gst_vaapi_display_destroy (GstVaapiDisplay * display)
 
   g_free (priv->vendor_string);
   priv->vendor_string = NULL;
-
-  gst_vaapi_display_replace_internal (&priv->parent, NULL);
-
-  if (priv->cache) {
-    gst_vaapi_display_cache_lock (priv->cache);
-    gst_vaapi_display_cache_remove (priv->cache, display);
-    gst_vaapi_display_cache_unlock (priv->cache);
-  }
-  gst_vaapi_display_cache_replace (&priv->cache, NULL);
-  free_display_cache ();
 }
 
 static gboolean
 gst_vaapi_display_create_unlocked (GstVaapiDisplay * display,
-    GstVaapiDisplayInitType init_type, gpointer init_value)
+    GstVaapiDisplayInitType init_type, gpointer data)
 {
   GstVaapiDisplayPrivate *const priv = GST_VAAPI_DISPLAY_GET_PRIVATE (display);
   const GstVaapiDisplayClass *const klass =
       GST_VAAPI_DISPLAY_GET_CLASS (display);
-  GstVaapiDisplayInfo info;
-  const GstVaapiDisplayInfo *cached_info = NULL;
-
-  memset (&info, 0, sizeof (info));
-  info.display = display;
-  info.display_type = priv->display_type;
+  GstVaapiDisplayInfo info = {
+    .display = display,
+  };
 
   switch (init_type) {
-    case GST_VAAPI_DISPLAY_INIT_FROM_VA_DISPLAY:
-      info.va_display = init_value;
-      priv->display = init_value;
+    case GST_VAAPI_DISPLAY_INIT_FROM_VA_DISPLAY:{
+      GstVaapiDisplayInfo *p_info = data;
+
+      info.va_display = p_info->va_display;
+      priv->display = p_info->va_display;
       priv->use_foreign_display = TRUE;
-      break;
+
+      if (!klass->bind_display)
+        break;
+
+      data = p_info->native_display;
+      goto bind_display;
+    }
     case GST_VAAPI_DISPLAY_INIT_FROM_DISPLAY_NAME:
-      if (klass->open_display && !klass->open_display (display, init_value))
+      if (klass->open_display && !klass->open_display (display, data))
         return FALSE;
       goto create_display;
     case GST_VAAPI_DISPLAY_INIT_FROM_NATIVE_DISPLAY:
-      if (klass->bind_display && !klass->bind_display (display, init_value))
+    bind_display:
+      if (klass->bind_display && !klass->bind_display (display, data))
         return FALSE;
       // fall-through
     create_display:
       if (!klass->get_display || !klass->get_display (display, &info))
         return FALSE;
       priv->display = info.va_display;
-      priv->display_type = info.display_type;
       priv->native_display = info.native_display;
-      if (klass->get_size)
-        klass->get_size (display, &priv->width, &priv->height);
-      if (klass->get_size_mm)
-        klass->get_size_mm (display, &priv->width_mm, &priv->height_mm);
-      gst_vaapi_display_calculate_pixel_aspect_ratio (display);
       break;
   }
   if (!priv->display)
     return FALSE;
 
-  cached_info = gst_vaapi_display_cache_lookup_by_va_display (priv->cache,
-      info.va_display);
-  if (cached_info) {
-    gst_vaapi_display_replace_internal (&priv->parent, cached_info->display);
-    priv->display_type = cached_info->display_type;
-  }
-
-  if (!priv->parent) {
-    if (!vaapi_initialize (priv->display))
-      return FALSE;
-  }
-
-  if (!cached_info) {
-    if (!gst_vaapi_display_cache_add (priv->cache, &info))
-      return FALSE;
-  }
+  if (!vaapi_initialize (priv->display))
+    return FALSE;
 
   GST_INFO_OBJECT (display, "new display addr=%p", display);
   g_free (priv->display_name);
@@ -955,20 +906,7 @@ static gboolean
 gst_vaapi_display_create (GstVaapiDisplay * display,
     GstVaapiDisplayInitType init_type, gpointer init_value)
 {
-  GstVaapiDisplayPrivate *const priv = GST_VAAPI_DISPLAY_GET_PRIVATE (display);
-  GstVaapiDisplayCache *cache;
-  gboolean success;
-
-  cache = get_display_cache ();
-  if (!cache)
-    return FALSE;
-  gst_vaapi_display_cache_replace (&priv->cache, cache);
-  gst_vaapi_display_cache_unref (cache);
-
-  gst_vaapi_display_cache_lock (cache);
-  success = gst_vaapi_display_create_unlocked (display, init_type, init_value);
-  gst_vaapi_display_cache_unlock (cache);
-  return success;
+  return gst_vaapi_display_create_unlocked (display, init_type, init_value);
 }
 
 static void
@@ -976,8 +914,6 @@ gst_vaapi_display_lock_default (GstVaapiDisplay * display)
 {
   GstVaapiDisplayPrivate *priv = GST_VAAPI_DISPLAY_GET_PRIVATE (display);
 
-  if (priv->parent)
-    priv = GST_VAAPI_DISPLAY_GET_PRIVATE (priv->parent);
   g_rec_mutex_lock (&priv->mutex);
 }
 
@@ -986,8 +922,6 @@ gst_vaapi_display_unlock_default (GstVaapiDisplay * display)
 {
   GstVaapiDisplayPrivate *priv = GST_VAAPI_DISPLAY_GET_PRIVATE (display);
 
-  if (priv->parent)
-    priv = GST_VAAPI_DISPLAY_GET_PRIVATE (priv->parent);
   g_rec_mutex_unlock (&priv->mutex);
 }
 
@@ -998,13 +932,128 @@ gst_vaapi_display_init (GstVaapiDisplay * display)
       gst_vaapi_display_get_instance_private (display);
 
   display->priv = priv;
-  priv->display_type = GST_VAAPI_DISPLAY_TYPE_ANY;
   priv->par_n = 1;
   priv->par_d = 1;
 
   g_rec_mutex_init (&priv->mutex);
 }
 
+static gboolean
+_set_property (GstVaapiDisplay * display, const GstVaapiProperty * prop,
+    const GValue * value)
+{
+  switch (prop->attribute.type) {
+    case VADisplayAttribRenderMode:{
+      GstVaapiRenderMode mode;
+      if (!G_VALUE_HOLDS (value, GST_VAAPI_TYPE_RENDER_MODE))
+        return FALSE;
+      mode = g_value_get_enum (value);
+      return gst_vaapi_display_set_render_mode (display, mode);
+    }
+    case VADisplayAttribRotation:{
+      GstVaapiRotation rotation;
+      if (!G_VALUE_HOLDS (value, GST_VAAPI_TYPE_ROTATION))
+        return FALSE;
+      rotation = g_value_get_enum (value);
+      return gst_vaapi_display_set_rotation (display, rotation);
+    }
+    case VADisplayAttribHue:
+    case VADisplayAttribSaturation:
+    case VADisplayAttribBrightness:
+    case VADisplayAttribContrast:{
+      gfloat v;
+      if (!G_VALUE_HOLDS (value, G_TYPE_FLOAT))
+        return FALSE;
+      v = g_value_get_float (value);
+      return set_color_balance (display, find_property_id (prop->name), v);
+    }
+    default:
+      break;
+  }
+
+  GST_WARNING ("unsupported property '%s'", prop->name);
+  return FALSE;
+}
+
+static void
+_gst_vaapi_display_set_property (GObject * object, guint property_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstVaapiDisplay *display = GST_VAAPI_DISPLAY (object);
+  const GstVaapiProperty *prop;
+
+  if (!ensure_properties (display))
+    return;
+
+  prop = find_property_by_pspec (display, pspec);
+  if (!prop) {
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    return;
+  }
+
+  _set_property (display, prop, value);
+}
+
+static gboolean
+_get_property (GstVaapiDisplay * display, const GstVaapiProperty * prop,
+    GValue * value)
+{
+  switch (prop->attribute.type) {
+    case VADisplayAttribRenderMode:{
+      GstVaapiRenderMode mode;
+      if (!gst_vaapi_display_get_render_mode (display, &mode))
+        return FALSE;
+      if (!G_IS_VALUE (value))
+        g_value_init (value, GST_VAAPI_TYPE_RENDER_MODE);
+      g_value_set_enum (value, mode);
+      break;
+    }
+    case VADisplayAttribRotation:{
+      GstVaapiRotation rotation;
+      rotation = gst_vaapi_display_get_rotation (display);
+      if (!G_IS_VALUE (value))
+        g_value_init (value, GST_VAAPI_TYPE_ROTATION);
+      g_value_set_enum (value, rotation);
+      break;
+    }
+    case VADisplayAttribHue:
+    case VADisplayAttribSaturation:
+    case VADisplayAttribBrightness:
+    case VADisplayAttribContrast:{
+      gfloat val;
+      if (!get_color_balance (display, find_property_id (prop->name), &val))
+        return FALSE;
+      if (!G_IS_VALUE (value))
+        g_value_init (value, G_TYPE_FLOAT);
+      g_value_set_float (value, val);
+      break;
+    }
+    default:
+      GST_WARNING ("unsupported property '%s'", prop->name);
+      return FALSE;
+  }
+  return TRUE;
+}
+
+static void
+_gst_vaapi_display_get_property (GObject * object, guint property_id,
+    GValue * value, GParamSpec * pspec)
+{
+  GstVaapiDisplay *display = GST_VAAPI_DISPLAY (object);
+  const GstVaapiProperty *prop;
+
+  if (!ensure_properties (display))
+    return;
+
+  prop = find_property_by_pspec (display, pspec);
+  if (!prop) {
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    return;
+  }
+
+  _get_property (display, prop, value);
+}
+
 static void
 gst_vaapi_display_finalize (GObject * object)
 {
@@ -1022,16 +1071,13 @@ gst_vaapi_display_class_init (GstVaapiDisplayClass * klass)
 {
   GObjectClass *const object_class = G_OBJECT_CLASS (klass);
 
-  libgstvaapi_init_once ();
-
   object_class->finalize = gst_vaapi_display_finalize;
+  object_class->set_property = _gst_vaapi_display_set_property;
+  object_class->get_property = _gst_vaapi_display_get_property;
+
   klass->lock = gst_vaapi_display_lock_default;
   klass->unlock = gst_vaapi_display_unlock_default;
-}
 
-static void
-gst_vaapi_display_properties_init (void)
-{
   /**
    * GstVaapiDisplay:render-mode:
    *
@@ -1096,6 +1142,8 @@ gst_vaapi_display_properties_init (void)
       g_param_spec_float (GST_VAAPI_DISPLAY_PROP_CONTRAST,
       "contrast",
       "The display contrast value", 0.0, 2.0, 1.0, G_PARAM_READWRITE);
+
+  g_object_class_install_properties (object_class, N_PROPERTIES, g_properties);
 }
 
 GstVaapiDisplay *
@@ -1128,18 +1176,12 @@ error:
 GstVaapiDisplay *
 gst_vaapi_display_new_with_display (VADisplay va_display)
 {
-  GstVaapiDisplayCache *const cache = get_display_cache ();
-  const GstVaapiDisplayInfo *info;
-
-  g_return_val_if_fail (va_display != NULL, NULL);
-  g_return_val_if_fail (cache != NULL, NULL);
-
-  info = gst_vaapi_display_cache_lookup_by_va_display (cache, va_display);
-  if (info)
-    return gst_vaapi_display_ref_internal (info->display);
+  GstVaapiDisplayInfo info = {
+    .va_display = va_display,
+  };
 
   return gst_vaapi_display_new (g_object_new (GST_TYPE_VAAPI_DISPLAY, NULL),
-      GST_VAAPI_DISPLAY_INIT_FROM_VA_DISPLAY, va_display);
+      GST_VAAPI_DISPLAY_INIT_FROM_VA_DISPLAY, &info);
 }
 
 /**
@@ -1351,6 +1393,8 @@ gst_vaapi_display_get_width (GstVaapiDisplay * display)
 {
   g_return_val_if_fail (display != NULL, 0);
 
+  gst_vaapi_display_ensure_screen_resolution (display);
+
   return GST_VAAPI_DISPLAY_GET_PRIVATE (display)->width;
 }
 
@@ -1367,6 +1411,8 @@ gst_vaapi_display_get_height (GstVaapiDisplay * display)
 {
   g_return_val_if_fail (display != NULL, 0);
 
+  gst_vaapi_display_ensure_screen_resolution (display);
+
   return GST_VAAPI_DISPLAY_GET_PRIVATE (display)->height;
 }
 
@@ -1384,6 +1430,8 @@ gst_vaapi_display_get_size (GstVaapiDisplay * display, guint * pwidth,
 {
   g_return_if_fail (GST_VAAPI_DISPLAY (display));
 
+  gst_vaapi_display_ensure_screen_resolution (display);
+
   if (pwidth)
     *pwidth = GST_VAAPI_DISPLAY_GET_PRIVATE (display)->width;
 
@@ -1405,6 +1453,8 @@ gst_vaapi_display_get_pixel_aspect_ratio (GstVaapiDisplay * display,
 {
   g_return_if_fail (display != NULL);
 
+  gst_vaapi_display_ensure_screen_resolution (display);
+
   if (par_n)
     *par_n = GST_VAAPI_DISPLAY_GET_PRIVATE (display)->par_n;
 
@@ -1690,38 +1740,7 @@ gst_vaapi_display_get_property (GstVaapiDisplay * display, const gchar * name,
   if (!prop)
     return FALSE;
 
-  switch (prop->attribute.type) {
-    case VADisplayAttribRenderMode:{
-      GstVaapiRenderMode mode;
-      if (!gst_vaapi_display_get_render_mode (display, &mode))
-        return FALSE;
-      g_value_init (out_value, GST_VAAPI_TYPE_RENDER_MODE);
-      g_value_set_enum (out_value, mode);
-      break;
-    }
-    case VADisplayAttribRotation:{
-      GstVaapiRotation rotation;
-      rotation = gst_vaapi_display_get_rotation (display);
-      g_value_init (out_value, GST_VAAPI_TYPE_ROTATION);
-      g_value_set_enum (out_value, rotation);
-      break;
-    }
-    case VADisplayAttribHue:
-    case VADisplayAttribSaturation:
-    case VADisplayAttribBrightness:
-    case VADisplayAttribContrast:{
-      gfloat value;
-      if (!get_color_balance (display, find_property_id (name), &value))
-        return FALSE;
-      g_value_init (out_value, G_TYPE_FLOAT);
-      g_value_set_float (out_value, value);
-      break;
-    }
-    default:
-      GST_WARNING ("unsupported property '%s'", name);
-      return FALSE;
-  }
-  return TRUE;
+  return _get_property (display, prop, out_value);
 }
 
 gboolean
@@ -1742,37 +1761,7 @@ gst_vaapi_display_set_property (GstVaapiDisplay * display, const gchar * name,
   if (!prop)
     return FALSE;
 
-  switch (prop->attribute.type) {
-    case VADisplayAttribRenderMode:{
-      GstVaapiRenderMode mode;
-      if (!G_VALUE_HOLDS (value, GST_VAAPI_TYPE_RENDER_MODE))
-        return FALSE;
-      mode = g_value_get_enum (value);
-      return gst_vaapi_display_set_render_mode (display, mode);
-    }
-    case VADisplayAttribRotation:{
-      GstVaapiRotation rotation;
-      if (!G_VALUE_HOLDS (value, GST_VAAPI_TYPE_ROTATION))
-        return FALSE;
-      rotation = g_value_get_enum (value);
-      return gst_vaapi_display_set_rotation (display, rotation);
-    }
-    case VADisplayAttribHue:
-    case VADisplayAttribSaturation:
-    case VADisplayAttribBrightness:
-    case VADisplayAttribContrast:{
-      gfloat v;
-      if (!G_VALUE_HOLDS (value, G_TYPE_FLOAT))
-        return FALSE;
-      v = g_value_get_float (value);
-      return set_color_balance (display, find_property_id (name), v);
-    }
-    default:
-      break;
-  }
-
-  GST_WARNING ("unsupported property '%s'", name);
-  return FALSE;
+  return _set_property (display, prop, value);
 }
 
 static gboolean
@@ -1852,9 +1841,7 @@ get_render_mode_VADisplayAttribDirectSurface (GstVaapiDisplay * display,
 static gboolean
 get_render_mode_default (GstVaapiDisplay * display, GstVaapiRenderMode * pmode)
 {
-  GstVaapiDisplayPrivate *const priv = GST_VAAPI_DISPLAY_GET_PRIVATE (display);
-
-  switch (priv->display_type) {
+  switch (GST_VAAPI_DISPLAY_VADISPLAY_TYPE (display)) {
 #if USE_WAYLAND
     case GST_VAAPI_DISPLAY_TYPE_WAYLAND:
       /* wl_buffer mapped from VA surface through vaGetSurfaceBufferWl() */
