@@ -1852,6 +1852,8 @@ ensure_misc_params (GstVaapiEncoderH265 * encoder, GstVaapiEncPicture * picture)
 
   if (!gst_vaapi_encoder_ensure_param_control_rate (base_encoder, picture))
     return FALSE;
+  if (!gst_vaapi_encoder_ensure_param_roi_regions (base_encoder, picture))
+    return FALSE;
   if (!gst_vaapi_encoder_ensure_param_quality_level (base_encoder, picture))
     return FALSE;
   return TRUE;
@@ -2024,8 +2026,6 @@ reset_properties (GstVaapiEncoderH265 * encoder)
 
   if (encoder->idr_period < base_encoder->keyframe_period)
     encoder->idr_period = base_encoder->keyframe_period;
-  if (encoder->idr_period > MAX_IDR_PERIOD)
-    encoder->idr_period = MAX_IDR_PERIOD;
 
   if (encoder->min_qp > encoder->init_qp)
     encoder->min_qp = encoder->init_qp;
