@@ -21,17 +21,14 @@
  *  Boston, MA 02110-1301 USA
  */
 
-#include "sysdeps.h"
 #include <math.h>
-#include <va/va.h>
-#include <va/va_enc_mpeg2.h>
+#include "sysdeps.h"
 #include <gst/base/gstbitwriter.h>
 #include "gstvaapicompat.h"
 #include "gstvaapiencoder_mpeg2.h"
 #include "gstvaapiencoder_mpeg2_priv.h"
 #include "gstvaapiutils_mpeg2_priv.h"
 #include "gstvaapicodedbufferproxy_priv.h"
-
 #include "gstvaapicontext.h"
 #include "gstvaapisurface.h"
 #include "gstvaapidisplay_priv.h"
@@ -103,7 +100,8 @@ ensure_hw_profile (GstVaapiEncoderMpeg2 * encoder)
   /* ERRORS */
 error_unsupported_profile:
   {
-    GST_ERROR ("unsupported HW profile (0x%08x)", encoder->profile);
+    GST_ERROR ("unsupported HW profile %s",
+        gst_vaapi_profile_get_name (encoder->profile));
     return FALSE;
   }
 }
@@ -451,51 +449,38 @@ ensure_picture (GstVaapiEncoderMpeg2 * encoder, GstVaapiEncPicture * picture,
   return TRUE;
 }
 
+static gboolean
+ensure_control_rate_params (GstVaapiEncoderMpeg2 * encoder)
+{
+  GstVaapiEncoder *const base_encoder = GST_VAAPI_ENCODER_CAST (encoder);
+
+  if (GST_VAAPI_ENCODER_RATE_CONTROL (encoder) == GST_VAAPI_RATECONTROL_CQP)
+    return TRUE;
+
+  /* RateControl params */
+  GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).initial_qp = encoder->cqp;
+
+  /* *INDENT-OFF* */
+  /* HRD params */
+  GST_VAAPI_ENCODER_VA_HRD (encoder) = (VAEncMiscParameterHRD) {
+    .buffer_size = base_encoder->bitrate * 1000 * 8,
+    .initial_buffer_fullness = base_encoder->bitrate * 1000 * 4,
+  };
+  /* *INDENT-ON* */
+
+  return TRUE;
+}
+
 static gboolean
 set_misc_parameters (GstVaapiEncoderMpeg2 * encoder,
     GstVaapiEncPicture * picture)
 {
   GstVaapiEncoder *const base_encoder = GST_VAAPI_ENCODER_CAST (encoder);
-  GstVaapiEncMiscParam *misc = NULL;
-  VAEncMiscParameterHRD *hrd;
-  VAEncMiscParameterRateControl *rate_control;
 
-  /* add hrd */
-  misc = GST_VAAPI_ENC_MISC_PARAM_NEW (HRD, encoder);
-  g_assert (misc);
-  if (!misc)
+  if (!gst_vaapi_encoder_ensure_param_control_rate (base_encoder, picture))
+    return FALSE;
+  if (!gst_vaapi_encoder_ensure_param_quality_level (base_encoder, picture))
     return FALSE;
-  gst_vaapi_enc_picture_add_misc_param (picture, misc);
-  hrd = misc->data;
-  if (base_encoder->bitrate > 0) {
-    hrd->initial_buffer_fullness = base_encoder->bitrate * 1000 * 4;
-    hrd->buffer_size = base_encoder->bitrate * 1000 * 8;
-  } else {
-    hrd->initial_buffer_fullness = 0;
-    hrd->buffer_size = 0;
-  }
-  gst_vaapi_codec_object_replace (&misc, NULL);
-
-  /* add ratecontrol */
-  if (GST_VAAPI_ENCODER_RATE_CONTROL (encoder) == GST_VAAPI_RATECONTROL_CBR) {
-    misc = GST_VAAPI_ENC_MISC_PARAM_NEW (RateControl, encoder);
-    g_assert (misc);
-    if (!misc)
-      return FALSE;
-    gst_vaapi_enc_picture_add_misc_param (picture, misc);
-    rate_control = misc->data;
-    memset (rate_control, 0, sizeof (VAEncMiscParameterRateControl));
-    if (base_encoder->bitrate)
-      rate_control->bits_per_second = base_encoder->bitrate * 1000;
-    else
-      rate_control->bits_per_second = 0;
-    rate_control->target_percentage = 70;
-    rate_control->window_size = 500;
-    rate_control->initial_qp = encoder->cqp;
-    rate_control->min_qp = 0;
-    rate_control->basic_unit_size = 0;
-    gst_vaapi_codec_object_replace (&misc, NULL);
-  }
   return TRUE;
 }
 
@@ -720,6 +705,7 @@ gst_vaapi_encoder_mpeg2_reconfigure (GstVaapiEncoder * base_encoder)
 
   if (!ensure_bitrate (encoder))
     goto error;
+  ensure_control_rate_params (encoder);
   return set_context_info (base_encoder);
 
   /* ERRORS */
