@@ -46,9 +46,6 @@
 /* Define the maximum value for view-id */
 #define MAX_VIEW_ID 1023
 
-/* Define the maximum IDR period */
-#define MAX_IDR_PERIOD 512
-
 /* Default CPB length (in milliseconds) */
 #define DEFAULT_CPB_LENGTH 1500
 
