@@ -47,8 +47,15 @@
 #if USE_VP9_ENCODER
 #include "gstvaapiencode_vp9.h"
 #endif
+
+#if USE_H264_FEI_ENCODER
+#include "gstvaapiencode_h264_fei.h"
 #endif
 
+#endif
+
+gboolean _gst_vaapi_has_video_processing = FALSE;
+
 #define PLUGIN_NAME     "vaapi"
 #define PLUGIN_DESC     "VA-API based elements"
 #define PLUGIN_LICENSE  "LGPL"
@@ -60,7 +67,7 @@ plugin_add_dependencies (GstPlugin * plugin)
     "DISPLAY", "WAYLAND_DISPLAY", NULL
   };
   const gchar *kernel_paths[] = { "/dev/dri", NULL };
-  const gchar *kernel_names[] = { "card", "render" };
+  const gchar *kernel_names[] = { "card", "render", NULL };
 
   /* features get updated upon changes in /dev/dri/card* */
   gst_plugin_add_dependency (plugin, NULL, kernel_paths, kernel_names,
@@ -186,6 +193,14 @@ gst_vaapiencode_register (GstPlugin * plugin, GstVaapiDisplay * display)
     }
   }
 
+#if USE_H264_FEI_ENCODER
+  if (gst_vaapi_display_has_encoder (display,
+          GST_VAAPI_PROFILE_H264_MAIN, GST_VAAPI_ENTRYPOINT_SLICE_ENCODE_FEI)) {
+    gst_element_register (plugin, "vaapih264feienc",
+        GST_RANK_SECONDARY, GST_TYPE_VAAPIENCODE_H264_FEI);
+  }
+#endif
+
   g_array_unref (codecs);
 }
 #endif
@@ -195,6 +210,7 @@ plugin_init (GstPlugin * plugin)
 {
   GstVaapiDisplay *display;
   GArray *decoders;
+  guint rank;
 
   plugin_add_dependencies (plugin);
 
@@ -204,22 +220,26 @@ plugin_init (GstPlugin * plugin)
   if (!gst_vaapi_driver_is_whitelisted (display))
     goto unsupported_driver;
 
+  _gst_vaapi_has_video_processing =
+      gst_vaapi_display_has_video_processing (display);
+
   decoders = display_get_decoder_codecs (display);
   if (decoders) {
     gst_vaapidecode_register (plugin, decoders);
     g_array_unref (decoders);
   }
 
-  if (gst_vaapi_display_has_video_processing (display)) {
-    gst_element_register (plugin, "vaapipostproc",
-        GST_RANK_PRIMARY, GST_TYPE_VAAPIPOSTPROC);
+  gst_element_register (plugin, "vaapipostproc",
+      GST_RANK_PRIMARY, GST_TYPE_VAAPIPOSTPROC);
 
-    gst_element_register (plugin, "vaapidecodebin",
-        GST_RANK_PRIMARY + 2, GST_TYPE_VAAPI_DECODE_BIN);
-  }
+  gst_element_register (plugin, "vaapidecodebin",
+      GST_RANK_PRIMARY + 2, GST_TYPE_VAAPI_DECODE_BIN);
+
+  rank = GST_RANK_PRIMARY;
+  if (g_getenv ("WAYLAND_DISPLAY"))
+    rank = GST_RANK_MARGINAL;
+  gst_element_register (plugin, "vaapisink", rank, GST_TYPE_VAAPISINK);
 
-  gst_element_register (plugin, "vaapisink",
-      GST_RANK_PRIMARY, GST_TYPE_VAAPISINK);
 #if USE_ENCODERS
   gst_vaapiencode_register (plugin, display);
 #endif
