@@ -39,6 +39,9 @@
 #if USE_WAYLAND
 # include <gst/vaapi/gstvaapidisplay_wayland.h>
 #endif
+#if USE_GST_GL_HELPERS
+# include <gst/gl/gl.h>
+#endif
 #include "gstvaapipluginutil.h"
 #include "gstvaapipluginbase.h"
 
@@ -245,6 +248,7 @@ gst_vaapi_find_gl_context (GstElement * element)
 {
   GstObject *gl_context;
   GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (element);
+  GstPadDirection direction = GST_PAD_UNKNOWN;
 
   /* if the element is vaapisink or any vaapi encoder it doesn't need
    * to know a GstGLContext in order to create an appropriate
@@ -254,11 +258,13 @@ gst_vaapi_find_gl_context (GstElement * element)
     return;
 
   gl_context = NULL;
-  if (!gst_vaapi_find_gl_local_context (element, &gl_context))
+  if (!gst_vaapi_find_gl_local_context (element, &gl_context, &direction))
     gl_context = gst_vaapi_plugin_base_create_gl_context (plugin);
 
   if (gl_context) {
     gst_vaapi_plugin_base_set_gl_context (plugin, gl_context);
+    if (direction == GST_PAD_SRC)
+      gst_vaapi_plugin_base_set_srcpad_can_dmabuf (plugin, gl_context);
     gst_object_unref (gl_context);
   }
 }
@@ -539,6 +545,7 @@ gst_vaapi_find_preferred_caps_feature (GstPad * pad, GstCaps * allowed_caps,
   guint i, j, num_structures;
   GstCaps *out_caps, *caps = NULL;
   static const guint feature_list[] = { GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE,
+    GST_VAAPI_CAPS_FEATURE_DMABUF,
     GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META,
     GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY,
   };
@@ -626,6 +633,9 @@ gst_vaapi_caps_feature_to_string (GstVaapiCapsFeature feature)
     case GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META:
       str = GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META;
       break;
+    case GST_VAAPI_CAPS_FEATURE_DMABUF:
+      str = GST_CAPS_FEATURE_MEMORY_DMABUF;
+      break;
     case GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE:
       str = GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE;
       break;
@@ -783,12 +793,32 @@ gst_video_info_force_nv12_if_encoded (GstVideoInfo * vinfo)
  * supported features.
  *
  * Returns: a new #GstVaapiDisplay instances. Free with
- * gst_vaapi_display_unref () after use.
+ * gst_vaapi_display_unref () after use. Or %NULL if no VA display is
+ * available.
  **/
 GstVaapiDisplay *
 gst_vaapi_create_test_display (void)
 {
-  return gst_vaapi_create_display (GST_VAAPI_DISPLAY_TYPE_ANY, NULL);
+  guint i;
+  GstVaapiDisplay *display = NULL;
+  const GstVaapiDisplayType test_display_map[] = {
+#if USE_DRM
+    GST_VAAPI_DISPLAY_TYPE_DRM,
+#endif
+#if USE_WAYLAND
+    GST_VAAPI_DISPLAY_TYPE_WAYLAND,
+#endif
+#if USE_X11
+    GST_VAAPI_DISPLAY_TYPE_X11,
+#endif
+  };
+
+  for (i = 0; i < G_N_ELEMENTS (test_display_map); i++) {
+    display = gst_vaapi_create_display (test_display_map[i], NULL);
+    if (display)
+      break;
+  }
+  return display;
 }
 
 /**
@@ -806,7 +836,7 @@ gst_vaapi_driver_is_whitelisted (GstVaapiDisplay * display)
   guint i;
   static const gchar *whitelist[] = {
     "Intel i965 driver",
-    "mesa gallium vaapi",
+    "mesa gallium",
     NULL
   };
 
