@@ -448,9 +448,11 @@ gst_vaapisink_x11_pre_start_event_thread (GstVaapiSink * sink)
 {
   GstVaapiDisplayX11 *const display =
       GST_VAAPI_DISPLAY_X11 (GST_VAAPI_PLUGIN_BASE_DISPLAY (sink));
-  static const int x11_event_mask = (KeyPressMask | KeyReleaseMask |
-      ButtonPressMask | ButtonReleaseMask | PointerMotionMask |
-      ExposureMask | StructureNotifyMask);
+  int x11_event_mask = (KeyPressMask | KeyReleaseMask |
+      PointerMotionMask | ExposureMask | StructureNotifyMask);
+
+  if (!sink->foreign_window)
+      x11_event_mask |= ButtonPressMask | ButtonReleaseMask;
 
   if (sink->window) {
     gst_vaapi_display_lock (GST_VAAPI_DISPLAY (display));
@@ -652,14 +654,14 @@ cb_get_gvalue (GstVaapiSink * sink, guint id)
 }
 
 static gboolean
-cb_set_gvalue (GstVaapiSink * sink, guint id, const GValue * value)
+cb_set_value (GstVaapiSink * sink, guint id, gfloat value)
 {
   GValue *const v_value = cb_get_gvalue (sink, id);
 
   if (!v_value)
     return FALSE;
 
-  g_value_set_float (v_value, g_value_get_float (value));
+  g_value_set_float (v_value, value);
   sink->cb_changed |= (1U << id);
   return TRUE;
 }
@@ -672,57 +674,46 @@ cb_get_value (GstVaapiSink * sink, guint id)
   return v_value ? g_value_get_float (v_value) : 0.0;
 }
 
-static gboolean
-cb_set_value (GstVaapiSink * sink, guint id, gfloat value)
-{
-  GValue v_value = G_VALUE_INIT;
-  gboolean success;
-
-  g_value_init (&v_value, G_TYPE_FLOAT);
-  g_value_set_float (&v_value, value);
-  success = cb_set_gvalue (sink, id, &v_value);
-  g_value_unset (&v_value);
-  return success;
-}
-
 static gboolean
 cb_sync_values_from_display (GstVaapiSink * sink, GstVaapiDisplay * display)
 {
-  GValue v_value = G_VALUE_INIT;
-  guint i, failures = 0;
+  guint i;
+  gfloat value;
 
   for (i = 0; i < G_N_ELEMENTS (sink->cb_values); i++) {
     const guint cb_id = CB_HUE + i;
-    if (!gst_vaapi_display_has_property (display, cb_map[i].prop_name))
+    if (!gst_vaapi_display_has_property (display, cb_map[i].prop_name)) {
+      GST_INFO_OBJECT (sink, "backend does not handle %s", cb_map[i].prop_name);
       continue;
+    }
 
-    if (G_IS_VALUE (&v_value))
-      g_value_unset (&v_value);
-    if (gst_vaapi_display_get_property (display, cb_map[i].prop_name, &v_value))
-      cb_set_gvalue (sink, cb_id, &v_value);
-    else
-      failures++;
+    value = 0.0;
+    g_object_get (display, cb_map[i].prop_name, &value, NULL);
+    cb_set_value (sink, cb_id, value);
   }
   sink->cb_changed = 0;
-  return failures == 0;
+  return TRUE;
 }
 
 static gboolean
 cb_sync_values_to_display (GstVaapiSink * sink, GstVaapiDisplay * display)
 {
-  guint i, failures = 0;
+  guint i;
 
   for (i = 0; i < G_N_ELEMENTS (sink->cb_values); i++) {
     const guint cb_id = CB_HUE + i;
     if (!(sink->cb_changed & (1U << cb_id)))
       continue;
+    if (!gst_vaapi_display_has_property (display, cb_map[i].prop_name)) {
+      GST_INFO_OBJECT (sink, "backend does not handle %s", cb_map[i].prop_name);
+      continue;
+    }
 
-    if (!gst_vaapi_display_set_property (display, cb_map[i].prop_name,
-            cb_get_gvalue (sink, cb_id)))
-      failures++;
+    g_object_set_property (G_OBJECT (display), cb_map[i].prop_name,
+        cb_get_gvalue (sink, cb_id));
   }
   sink->cb_changed = 0;
-  return failures == 0;
+  return TRUE;
 }
 
 #define CB_CHANNEL_FACTOR (1000.0)
@@ -1325,9 +1316,6 @@ gst_vaapisink_set_caps (GstBaseSink * base_sink, GstCaps * caps)
     return FALSE;
   display = GST_VAAPI_PLUGIN_BASE_DISPLAY (sink);
 
-  if (GST_VAAPI_PLUGIN_BASE_DISPLAY_TYPE (sink) == GST_VAAPI_DISPLAY_TYPE_DRM)
-    return TRUE;
-
   if (!gst_vaapi_plugin_base_set_caps (plugin, caps, NULL))
     return FALSE;
 
@@ -1346,6 +1334,9 @@ gst_vaapisink_set_caps (GstBaseSink * base_sink, GstCaps * caps)
   gst_vaapisink_ensure_colorbalance (sink);
   gst_vaapisink_ensure_rotation (sink, FALSE);
 
+  if (GST_VAAPI_PLUGIN_BASE_DISPLAY_TYPE (sink) == GST_VAAPI_DISPLAY_TYPE_DRM)
+    return TRUE;
+
   gst_vaapisink_ensure_window_size (sink, &win_width, &win_height);
   if (sink->window) {
     if (!sink->foreign_window || sink->fullscreen)
@@ -1409,8 +1400,9 @@ gst_vaapisink_show_frame_unlocked (GstVaapiSink * sink, GstBuffer * src_buffer)
     return ret;
 
   meta = gst_buffer_get_vaapi_video_meta (buffer);
-  GST_VAAPI_PLUGIN_BASE_DISPLAY_REPLACE (sink,
-      gst_vaapi_video_meta_get_display (meta));
+  if (gst_vaapi_video_meta_get_display (meta) !=
+      GST_VAAPI_PLUGIN_BASE_DISPLAY (sink))
+    goto different_display;
 
   proxy = gst_vaapi_video_meta_get_surface_proxy (meta);
   if (!proxy)
@@ -1492,6 +1484,13 @@ no_surface:
     ret = GST_FLOW_ERROR;
     goto done;
   }
+
+different_display:
+  {
+    GST_WARNING_OBJECT (sink, "incoming surface has different VAAPI Display");
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
 }
 
 static GstFlowReturn
@@ -1594,7 +1593,8 @@ gst_vaapisink_set_property (GObject * object,
     case PROP_SATURATION:
     case PROP_BRIGHTNESS:
     case PROP_CONTRAST:
-      cb_set_gvalue (sink, (prop_id - PROP_HUE) + CB_HUE, value);
+      cb_set_value (sink, (prop_id - PROP_HUE) + CB_HUE,
+          g_value_get_float (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
