@@ -49,6 +49,7 @@ struct _GstVaapiVideoBufferPoolPrivate
   GstVaapiDisplay *display;
   guint options;
   guint use_dmabuf_memory:1;
+  guint forced_video_meta:1;
 };
 
 #define GST_VAAPI_VIDEO_BUFFER_POOL_GET_PRIVATE(obj) \
@@ -142,7 +143,7 @@ gst_vaapi_video_buffer_pool_set_config (GstBufferPool * pool,
   const GstVideoInfo *negotiated_vinfo;
   GstVideoAlignment align;
   GstAllocator *allocator;
-  gboolean ret, updated = FALSE;
+  gboolean ret;
   guint size, min_buffers, max_buffers;
   guint surface_alloc_flags;
 
@@ -248,17 +249,17 @@ gst_vaapi_video_buffer_pool_set_config (GstBufferPool * pool,
   if (gst_buffer_pool_config_has_option (config,
           GST_BUFFER_POOL_OPTION_VIDEO_META)) {
     priv->options |= GST_VAAPI_VIDEO_BUFFER_POOL_OPTION_VIDEO_META;
-  } else {
+  } else if (gst_caps_is_video_raw (caps) && !priv->use_dmabuf_memory) {
     gint i;
+
     for (i = 0; i < GST_VIDEO_INFO_N_PLANES (&new_allocation_vinfo); i++) {
       if (GST_VIDEO_INFO_PLANE_OFFSET (&new_allocation_vinfo, i) !=
           GST_VIDEO_INFO_PLANE_OFFSET (&priv->vmeta_vinfo, i) ||
           GST_VIDEO_INFO_PLANE_STRIDE (&new_allocation_vinfo, i) !=
           GST_VIDEO_INFO_PLANE_STRIDE (&priv->vmeta_vinfo, i)) {
         priv->options |= GST_VAAPI_VIDEO_BUFFER_POOL_OPTION_VIDEO_META;
-        gst_buffer_pool_config_add_option (config,
-            GST_BUFFER_POOL_OPTION_VIDEO_META);
-        updated = TRUE;
+        priv->forced_video_meta = TRUE;
+        GST_INFO_OBJECT (pool, "adding unrequested video meta");
         break;
       }
     }
@@ -277,7 +278,7 @@ gst_vaapi_video_buffer_pool_set_config (GstBufferPool * pool,
   ret =
       GST_BUFFER_POOL_CLASS
       (gst_vaapi_video_buffer_pool_parent_class)->set_config (pool, config);
-  return !updated && ret;
+  return ret;
 
   /* ERRORS */
 error_invalid_config:
@@ -536,3 +537,25 @@ gst_vaapi_video_buffer_pool_new (GstVaapiDisplay * display)
   return g_object_new (GST_VAAPI_TYPE_VIDEO_BUFFER_POOL,
       "display", display, NULL);
 }
+
+/**
+ * gst_vaapi_video_buffer_pool_copy_buffer:
+ * @pool: a #GstVaapiVideoBufferPool
+ *
+ * Returns if the @pool force set of #GstVideoMeta. If so, the element
+ * should copy the generated buffer by the pool to a system allocated
+ * buffer. Otherwise, downstream could not display correctly the
+ * frame.
+ *
+ * Returns: %TRUE if #GstVideoMeta is forced.
+ **/
+gboolean
+gst_vaapi_video_buffer_pool_copy_buffer (GstBufferPool * pool)
+{
+  GstVaapiVideoBufferPoolPrivate *priv;
+
+  g_return_val_if_fail (GST_VAAPI_IS_VIDEO_BUFFER_POOL (pool), FALSE);
+
+  priv = GST_VAAPI_VIDEO_BUFFER_POOL_GET_PRIVATE (pool);
+  return priv->forced_video_meta;
+}
