@@ -28,6 +28,9 @@
 #if USE_GST_GL_HELPERS
 # include <gst/gl/gl.h>
 #endif
+#if USE_X11
+#include <gst/vaapi/gstvaapidisplay_x11.h>
+#endif
 
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_CONTEXT);
 
@@ -69,21 +72,53 @@ gst_vaapi_video_context_new_with_display (GstVaapiDisplay * display,
 }
 
 gboolean
-gst_vaapi_video_context_get_display (GstContext * context,
+gst_vaapi_video_context_get_display (GstContext * context, gboolean app_context,
     GstVaapiDisplay ** display_ptr)
 {
   const GstStructure *structure;
   const gchar *type;
+  GstVaapiDisplay *display = NULL;
 
   g_return_val_if_fail (GST_IS_CONTEXT (context), FALSE);
 
   type = gst_context_get_context_type (context);
-  if (g_strcmp0 (type, GST_VAAPI_DISPLAY_CONTEXT_TYPE_NAME))
-    return FALSE;
 
-  structure = gst_context_get_structure (context);
-  return gst_structure_get (structure, GST_VAAPI_DISPLAY_CONTEXT_TYPE_NAME,
-      GST_TYPE_VAAPI_DISPLAY, display_ptr, NULL);
+  if (!g_strcmp0 (type, GST_VAAPI_DISPLAY_CONTEXT_TYPE_NAME)) {
+    structure = gst_context_get_structure (context);
+    return gst_structure_get (structure, GST_VAAPI_DISPLAY_CONTEXT_TYPE_NAME,
+        GST_TYPE_VAAPI_DISPLAY, display_ptr, NULL);
+  }
+
+  if (app_context && !g_strcmp0 (type, GST_VAAPI_DISPLAY_APP_CONTEXT_TYPE_NAME)) {
+    VADisplay va_display = NULL;
+    structure = gst_context_get_structure (context);
+
+    if (gst_structure_get (structure, "va-display", G_TYPE_POINTER, &va_display,
+            NULL)) {
+#if USE_X11
+      Display *x11_display = NULL;
+      if (gst_structure_get (structure, "x11-display", G_TYPE_POINTER,
+              &x11_display, NULL)) {
+        display =
+            gst_vaapi_display_x11_new_with_va_display (va_display, x11_display);
+      }
+#endif
+
+      _init_context_debug ();
+
+      if (!display) {
+        GST_CAT_WARNING (GST_CAT_CONTEXT,
+            "Cannot create GstVaapiDisplay if only VADisplay is provided");
+        return FALSE;
+      }
+      GST_CAT_INFO (GST_CAT_CONTEXT,
+          "new display with context %" GST_PTR_FORMAT, display);
+      *display_ptr = display;
+      return TRUE;
+    }
+  }
+
+  return FALSE;
 }
 
 static gboolean
@@ -187,6 +222,32 @@ found:
   gst_query_unref (query);
 }
 
+static gboolean
+_gst_vaapi_sink_find_context (GstElement * element)
+{
+  GstQuery *query;
+  GstMessage *msg;
+  gboolean found;
+
+  /* 1. Query upstream for an already created GstVaapiDisplay */
+  query = gst_query_new_context (GST_VAAPI_DISPLAY_CONTEXT_TYPE_NAME);
+  found = _gst_context_get_from_query (element, query, GST_PAD_SINK);
+  gst_query_unref (query);
+  if (found)
+    return TRUE;
+
+  /* 2. Post a GST_MESSAGE_NEED_CONTEXT message on the bus with a
+   * gst.vaapi.app.Display context from the application */
+  msg = gst_message_new_need_context (GST_OBJECT_CAST (element),
+      GST_VAAPI_DISPLAY_APP_CONTEXT_TYPE_NAME);
+  if (!gst_element_post_message (element, msg)) {
+    _init_context_debug ();
+    GST_CAT_INFO_OBJECT (GST_CAT_CONTEXT, element, "No bus attached");
+  }
+
+  return FALSE;
+}
+
 gboolean
 gst_vaapi_video_context_prepare (GstElement * element,
     GstVaapiDisplay ** display_ptr)
@@ -198,16 +259,26 @@ gst_vaapi_video_context_prepare (GstElement * element,
    *     type.
    */
   if (*display_ptr) {
-    GST_LOG_OBJECT (element, "already have a display (%p)", *display_ptr);
+    GST_LOG_OBJECT (element, "already have a display %" GST_PTR_FORMAT,
+        *display_ptr);
     return TRUE;
   }
 
-  _gst_context_query (element, GST_VAAPI_DISPLAY_CONTEXT_TYPE_NAME);
+  if (GST_IS_VIDEO_SINK (element)) {
+    if (!_gst_vaapi_sink_find_context (element) && *display_ptr) {
+      /* Propagate if display was created from application */
+      gst_vaapi_video_context_propagate (element, *display_ptr);
+    }
+  } else {
+    _gst_context_query (element, GST_VAAPI_DISPLAY_CONTEXT_TYPE_NAME);
+  }
 
-  if (*display_ptr)
-    GST_LOG_OBJECT (element, "found a display (%p)", *display_ptr);
+  if (*display_ptr) {
+    GST_LOG_OBJECT (element, "found a display %" GST_PTR_FORMAT, *display_ptr);
+    return TRUE;
+  }
 
-  return *display_ptr != NULL;
+  return FALSE;
 }
 
 /* 5) Create a context by itself and post a GST_MESSAGE_HAVE_CONTEXT message
@@ -224,25 +295,39 @@ gst_vaapi_video_context_propagate (GstElement * element,
 
   _init_context_debug ();
   GST_CAT_INFO_OBJECT (GST_CAT_CONTEXT, element,
-      "posting `have-context' (%p) message with display (%p)",
+      "posting `have-context' (%p) message with display %" GST_PTR_FORMAT,
       context, display);
   msg = gst_message_new_have_context (GST_OBJECT_CAST (element), context);
   if (!gst_element_post_message (element, msg))
     GST_CAT_INFO_OBJECT (GST_CAT_CONTEXT, element, "No bus attached");
 }
 
+/**
+ * gst_vaapi_find_gl_local_context:
+ * @element: the #GstElement where the search begins
+ * @gl_context_ptr: the pointer where the GstGL context is going to be
+ * stored
+ * @direction_ptr: the pointer of the #GstPadDirection where the GstGL
+ * context was found
+ *
+ * Query the pipeline, downstream and upstream for a GstGL context
+ *
+ * Returns: %TRUE if found; otherwise %FALSE
+ **/
 gboolean
 gst_vaapi_find_gl_local_context (GstElement * element,
-    GstObject ** gl_context_ptr)
+    GstObject ** gl_context_ptr, GstPadDirection * direction_ptr)
 {
 #if USE_GST_GL_HELPERS
   GstQuery *query;
   GstContext *context;
   const GstStructure *s;
   GstObject *gl_context;
+  GstPadDirection direction;
 
   g_return_val_if_fail (gl_context_ptr, FALSE);
 
+  direction = GST_PAD_UNKNOWN;
   gl_context = NULL;
   query = gst_query_new_context ("gst.gl.local_context");
   if (_gst_context_run_query (element, query, GST_PAD_SRC)) {
@@ -250,6 +335,7 @@ gst_vaapi_find_gl_local_context (GstElement * element,
     if (context) {
       s = gst_context_get_structure (context);
       gst_structure_get (s, "context", GST_TYPE_GL_CONTEXT, &gl_context, NULL);
+      direction = GST_PAD_SRC;
     }
   }
   if (!gl_context && _gst_context_run_query (element, query, GST_PAD_SINK)) {
@@ -257,11 +343,13 @@ gst_vaapi_find_gl_local_context (GstElement * element,
     if (context) {
       s = gst_context_get_structure (context);
       gst_structure_get (s, "context", GST_TYPE_GL_CONTEXT, &gl_context, NULL);
+      direction = GST_PAD_SINK;
     }
   }
   gst_query_unref (query);
   if (gl_context) {
     *gl_context_ptr = gl_context;
+    *direction_ptr = direction;
     return TRUE;
   }
 #endif
