@@ -1159,27 +1159,40 @@ gst_vaapi_video_info_quark_get (void)
   return g_quark;
 }
 
-#define INFO_QUARK info_quark_get ()
+#define ALLOCATION_VINFO_QUARK allocation_vinfo_quark_get ()
 static GQuark
-info_quark_get (void)
+allocation_vinfo_quark_get (void)
 {
   static gsize g_quark;
 
   if (g_once_init_enter (&g_quark)) {
-    gsize quark = (gsize) g_quark_from_static_string ("info");
+    gsize quark = (gsize) g_quark_from_static_string ("allocation-vinfo");
     g_once_init_leave (&g_quark, quark);
   }
   return g_quark;
 }
 
-#define FLAGS_QUARK flags_quark_get ()
+#define SURFACE_ALLOC_FLAGS_QUARK surface_alloc_flags_quark_get ()
 static GQuark
-flags_quark_get (void)
+surface_alloc_flags_quark_get (void)
 {
   static gsize g_quark;
 
   if (g_once_init_enter (&g_quark)) {
-    gsize quark = (gsize) g_quark_from_static_string ("flags");
+    gsize quark = (gsize) g_quark_from_static_string ("surface-alloc-flags");
+    g_once_init_leave (&g_quark, quark);
+  }
+  return g_quark;
+}
+
+#define NEGOTIATED_VINFO_QUARK negotiated_vinfo_quark_get ()
+static GQuark
+negotiated_vinfo_quark_get (void)
+{
+  static gsize g_quark;
+
+  if (g_once_init_enter (&g_quark)) {
+    gsize quark = (gsize) g_quark_from_static_string ("negotiated-vinfo");
     g_once_init_leave (&g_quark, quark);
   }
   return g_quark;
@@ -1188,10 +1201,14 @@ flags_quark_get (void)
 /**
  * gst_allocator_get_vaapi_video_info:
  * @allocator: a #GstAllocator
- * @out_flags_ptr: (out): the stored flags
+ * @out_flags_ptr: (out): the stored surface allocation flags
  *
  * Will get the @allocator qdata to fetch the flags and the
- * #GstVideoInfo stored in it.
+ * allocation's #GstVideoInfo stored in it.
+ *
+ * The allocation video info, is the image video info in the case of
+ * the #GstVaapiVideoAllocator; and the allocation video info in the
+ * case of #GstVaapiDmaBufAllocator.
  *
  * Returns: the stored #GstVideoInfo
  **/
@@ -1210,13 +1227,13 @@ gst_allocator_get_vaapi_video_info (GstAllocator * allocator,
     return NULL;
 
   if (out_flags_ptr) {
-    value = gst_structure_id_get_value (structure, FLAGS_QUARK);
+    value = gst_structure_id_get_value (structure, SURFACE_ALLOC_FLAGS_QUARK);
     if (!value)
       return NULL;
     *out_flags_ptr = g_value_get_uint (value);
   }
 
-  value = gst_structure_id_get_value (structure, INFO_QUARK);
+  value = gst_structure_id_get_value (structure, ALLOCATION_VINFO_QUARK);
   if (!value)
     return NULL;
   return g_value_get_boxed (value);
@@ -1225,29 +1242,69 @@ gst_allocator_get_vaapi_video_info (GstAllocator * allocator,
 /**
  * gst_allocator_set_vaapi_video_info:
  * @allocator: a #GstAllocator
- * @vip: the #GstVideoInfo to store
- * @flags: the flags to store
+ * @alloc_info: the allocation #GstVideoInfo to store
+ * @surface_alloc_flags: the flags to store
  *
- * Stores as GObject's qdata the @vip and the @flags in the
- * allocator. This will "decorate" the allocator as a GstVaapi one.
+ * Stores as GObject's qdata the @alloc_info and the
+ * @surface_alloc_flags in the allocator. This will "decorate" the
+ * allocator as a GstVaapi one.
  *
  * Returns: always %TRUE
  **/
 gboolean
 gst_allocator_set_vaapi_video_info (GstAllocator * allocator,
-    const GstVideoInfo * vip, guint flags)
+    const GstVideoInfo * alloc_info, guint surface_alloc_flags)
 {
   g_return_val_if_fail (GST_IS_ALLOCATOR (allocator), FALSE);
-  g_return_val_if_fail (vip != NULL, FALSE);
+  g_return_val_if_fail (alloc_info != NULL, FALSE);
 
   g_object_set_qdata_full (G_OBJECT (allocator), GST_VAAPI_VIDEO_INFO_QUARK,
-      gst_structure_new_id (GST_VAAPI_VIDEO_INFO_QUARK, INFO_QUARK,
-          GST_TYPE_VIDEO_INFO, vip, FLAGS_QUARK, G_TYPE_UINT, flags, NULL),
+      gst_structure_new_id (GST_VAAPI_VIDEO_INFO_QUARK,
+          ALLOCATION_VINFO_QUARK, GST_TYPE_VIDEO_INFO, alloc_info,
+          SURFACE_ALLOC_FLAGS_QUARK, G_TYPE_UINT, surface_alloc_flags, NULL),
       (GDestroyNotify) gst_structure_free);
 
   return TRUE;
 }
 
+/**
+ * gst_allocator_set_vaapi_negotiated_video_info:
+ * @allocator: a #GstAllocator
+ * @negotiated_vinfo: the negotiated #GstVideoInfo to store
+ *
+ * Stores as GObject's qdata the @negotiated_vinfo in the allocator
+ * instance.
+ *
+ * The @negotiated_vinfo is different of the @alloc_info from
+ * gst_allocator_set_vaapi_video_info(), and might not be set.
+ **/
+void
+gst_allocator_set_vaapi_negotiated_video_info (GstAllocator * allocator,
+    const GstVideoInfo * negotiated_vinfo)
+{
+  g_return_if_fail (allocator && GST_IS_ALLOCATOR (allocator));
+  g_return_if_fail (negotiated_vinfo);
+
+  g_object_set_qdata_full (G_OBJECT (allocator), NEGOTIATED_VINFO_QUARK,
+      gst_video_info_copy (negotiated_vinfo),
+      (GDestroyNotify) gst_video_info_free);
+}
+
+/**
+ * gst_allocator_get_vaapi_negotiated_video_info:
+ * @allocator: a #GstAllocator
+ *
+ * Returns: the stored negotiation #GstVideoInfo, if it was stored
+ * previously. Otherwise, %NULL
+ **/
+GstVideoInfo *
+gst_allocator_get_vaapi_negotiated_video_info (GstAllocator * allocator)
+{
+  g_return_val_if_fail (GST_IS_ALLOCATOR (allocator), NULL);
+
+  return g_object_get_qdata (G_OBJECT (allocator), NEGOTIATED_VINFO_QUARK);
+}
+
 /**
  * gst_vaapi_is_dmabuf_allocator:
  * @allocator: an #GstAllocator
@@ -1270,3 +1327,50 @@ gst_vaapi_is_dmabuf_allocator (GstAllocator * allocator)
   st = g_object_get_qdata (G_OBJECT (allocator), GST_VAAPI_VIDEO_INFO_QUARK);
   return (st != NULL);
 }
+
+/**
+ * gst_vaapi_dmabuf_can_map:
+ * @display: a #GstVaapiDisplay
+ * @allocator: a #GstAllocator
+ *
+ * It will create a dmabuf-based buffer using @allocator, and it will
+ * try to map it using gst_memory_map().
+ *
+ * Returns: %TRUE if the internal dummy buffer can be
+ * mapped. Otherwise %FALSE.
+ **/
+gboolean
+gst_vaapi_dmabuf_can_map (GstVaapiDisplay * display, GstAllocator * allocator)
+{
+  GstVaapiVideoMeta *meta;
+  GstMemory *mem;
+  GstMapInfo info;
+  gboolean ret;
+
+  g_return_val_if_fail (display != NULL, FALSE);
+
+  ret = FALSE;
+  mem = NULL;
+  meta = NULL;
+  if (!gst_vaapi_is_dmabuf_allocator (allocator))
+    return FALSE;
+  meta = gst_vaapi_video_meta_new (display);
+  if (!meta)
+    return FALSE;
+  mem = gst_vaapi_dmabuf_memory_new (allocator, meta);
+  if (!mem)
+    goto bail;
+
+  if (!gst_memory_map (mem, &info, GST_MAP_READWRITE) || info.size == 0)
+    goto bail;
+
+  gst_memory_unmap (mem, &info);
+  ret = TRUE;
+
+bail:
+  if (mem)
+    gst_memory_unref (mem);
+  if (meta)
+    gst_vaapi_video_meta_unref (meta);
+  return ret;
+}
